@@ -1647,21 +1647,22 @@ void setcolormark(int colorval)
    /* Set GUI variables and execute any command tags associated */
    /* with the "color" command */
 
-   Tcl_Obj *objv[2];
+   Tcl_Obj *objv[3];
    int i;
 
    objv[0] = Tcl_NewStringObj("color", 5);
+   objv[1] = Tcl_NewStringObj("set", 3);
    if (colorval == DEFAULTCOLOR)
-      objv[1] = Tcl_NewStringObj("inherit", 7);
+      objv[2] = Tcl_NewStringObj("inherit", 7);
    else {
       for (i = 0; i < number_colors; i++)
          if (colorlist[i].color.pixel == colorval)
 	    break;
-      objv[1] = Tcl_NewIntObj((int)i);
+      objv[2] = Tcl_NewIntObj((int)i);
    }
-   Tcl_SetVar2Ex(xcinterp, "colorval", NULL, objv[1], TCL_NAMESPACE_ONLY);
+   Tcl_SetVar2Ex(xcinterp, "colorval", NULL, objv[2], TCL_NAMESPACE_ONLY);
 
-   XcTagCallback(xcinterp, 2, objv);
+   XcTagCallback(xcinterp, 3, objv);
 #endif
 }
 
