@@ -204,6 +204,29 @@ int get_num_params(objectptr thisobj)
    return nparam;
 }
 
+/*----------------------------------------------------------------------*/
+/* Remove an element parameter (eparam) and free memory associated with	*/
+/* the parameter key.							*/
+/*----------------------------------------------------------------------*/
+
+void free_element_param(genericptr thiselem, eparamptr thisepp)
+{
+   eparamptr epp, lastepp = NULL;
+
+   for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
+      if (epp == thisepp) {
+	 if (lastepp != NULL)
+	    lastepp->next = epp->next;
+	 else
+	    thiselem->passed = epp->next;
+	 free(epp->key);
+	 free(epp);
+	 break;
+      }
+      lastepp = epp; 
+   }
+}
+
 /*----------------------------------------------------------------------*/
 /* Free an instance parameter.  Note that this routine does not free	*/
 /* any strings associated with string parameters!			*/
@@ -1095,32 +1118,60 @@ void makenumericalp(genericptr *gelem, u_int mode)
 }
 
 /*--------------------------------------------------------------*/
-/* remove a numerical (integer or float) parameter		*/
+/* Remove a numerical (integer or float) parameter.  Remove by	*/
+/* type, rather than key.  There may be several keys associated	*/
+/* with a particular type, so we want to remove all of them.	*/
 /*--------------------------------------------------------------*/
 
 void unmakenumericalp(genericptr *gelem, u_int mode)
 {
+   genericptr *pgen;
    eparamptr epp;
+   oparamptr ops;
+   char *key;
+   Boolean done = False, is_last = True;
    
    /* Parameterized strings are handled by makeparam() */
-   if (((*gelem)->type == LABEL) && (mode == P_SUBSTRING)) {
-      unparameterize(mode);
-      setparammarks(*gelem);
+   if (mode == P_SUBSTRING) {
+      Fprintf(stderr, "Error: Unmakenumericalp called on a string parameter.\n");
       return;
    }
-   else {
 
-      /* (to be done) */
+   /* Avoid referencing the object by only looking at the element.	*/
+   /* But, avoid dereferencing the pointer!				*/
 
-/*
-      for (epp = gelem->passed; epp != NULL; epp = epp->next)
-	 if (epp->which == (u_char)mode)
-*/
-	    /* remove the eparam */
+   while (!done) {
+      key = NULL;
+      done = True;
+      for (epp = (*gelem)->passed; epp != NULL; epp = epp->next) {
+	 ops = match_param(topobject, epp->key);
+	 if (ops == NULL) break;	/* Error---no such parameter */
+         else if (ops->which == (u_char)mode) {
+	    key = ops->key;
+	    free_element_param(*gelem, epp);
 
-      /* check for any other references to the parameter.  If there	*/
-      /* are none, remove all instance records of the eparam, then	*/
-      /* remove the parameter itself from the object.			*/
+	    /* check for any other references to the parameter.  If there */
+	    /* are none, remove all instance records of the eparam, then  */
+	    /* remove the parameter itself from the object.		  */
+
+	    for (pgen = topobject->plist; pgen < topobject->plist
+			+ topobject->parts; pgen++) {
+	       if (*pgen == *gelem) continue;
+	       for (epp = (*pgen)->passed; epp != NULL; epp = epp->next) {
+		  if (!strcmp(epp->key, key)) {
+		     is_last = False;
+		     break;
+		  }
+	       }
+	       if (!is_last) break;
+	    }
+	    if (is_last)
+	       free_object_param(topobject, ops);
+
+	    done = False;
+	    break;
+	 }
+      }
    }
 }
 
@@ -1289,7 +1340,8 @@ void destroyinst(objinstptr tinst, objectptr refobj, char *key)
    if (tinst->thisobject == refobj) {
       ops = match_instance_param(tinst, key);
       if (ops != NULL) {
-	 freelabel(ops->parameter.string);
+	 if (ops->type == XC_STRING)
+	    freelabel(ops->parameter.string);
 	 free_instance_param(tinst, ops);
       }
    }
@@ -1315,6 +1367,63 @@ void searchinst(objectptr topobj, objectptr refobj, char *key)
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Destroy the object parameter with key "key" in the object	*/
+/* "thisobj".  This requires first tracking down and removing	*/
+/* all instances of the parameter which may exist anywhere in	*/
+/* the database.						*/
+/*--------------------------------------------------------------*/
+
+void free_object_param(objectptr thisobj, oparamptr thisparam)
+{
+   int k, j, l = -1;
+   liblistptr spec;
+   oparamptr ops, lastops = NULL;
+   char *key = thisparam->key;
+
+   /* Find all instances of this object and remove any parameter */
+   /* substitutions which may have been made.		         */
+
+   for (k = 0; k < xobjs.pages; k++) {
+      if (xobjs.pagelist[k]->pageinst != NULL)
+         searchinst(xobjs.pagelist[k]->pageinst->thisobject, thisobj, key);
+   }
+   for (j = 0; j < xobjs.numlibs; j++) {
+      for (k = 0; k < xobjs.userlibs[j].number; k++) {
+	 if (*(xobjs.userlibs[j].library + k) == thisobj)
+	    l = j;
+	 else
+            searchinst(*(xobjs.userlibs[j].library + k), thisobj, key);
+      }
+   }
+   for (k = 0; k < xobjs.delbuffer.number; k++) {
+      searchinst(*(xobjs.delbuffer.library + k), thisobj, key);
+   }
+
+   /* Also check through all instances on the library page */
+   if (l >= 0)
+      for (spec = xobjs.userlibs[l].instlist; spec != NULL; spec = spec->next) 
+         destroyinst(spec->thisinst, thisobj, key);
+
+   /* Remove the parameter from the object itself, tidying up	*/
+   /* the linked list after it.					*/
+
+   for (ops = thisobj->params; ops != NULL; ops = ops->next) {
+      if (ops == thisparam) {
+	 if (lastops != NULL)
+	    lastops->next = ops->next;
+	 else
+	    thisobj->params = ops->next;
+	 free(ops->key);
+	 free(ops);
+	 break;
+      }
+      lastops = ops; 
+   }
+   
+   incr_changes(thisobj);
+}
+
 /*--------------------------------------------------------------*/
 /* Check if this string contains a parameter			*/
 /*--------------------------------------------------------------*/
@@ -1337,10 +1446,8 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    genericptr *pgen;
    labelptr plab;
    oparamptr ops;
-   liblistptr spec;
    stringpart *strptr, *lastpart, *nextpart, *endpart, *newstr, *subs;
    char *key;
-   int j, k, l = -1;
    Boolean is_last = True;
    
    /* make sure there is a parameter here */
@@ -1409,37 +1516,10 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    mergestring(lastpart);
 
    redrawtext(thislabel);
-   if (!is_last) return;
-
-   /* Find all instances of this object and remove any parameter */
-   /* substitutions which may have been made.		         */
-
-   for (k = 0; k < xobjs.pages; k++) {
-      if (xobjs.pagelist[k]->pageinst != NULL)
-         searchinst(xobjs.pagelist[k]->pageinst->thisobject, topobject, key);
+   if (is_last) {
+      freelabel(subs);
+      free_object_param(topobject, ops);
    }
-   for (j = 0; j < xobjs.numlibs; j++) {
-      for (k = 0; k < xobjs.userlibs[j].number; k++) {
-	 if (*(xobjs.userlibs[j].library + k) == topobject)
-	    l = j;
-	 else
-            searchinst(*(xobjs.userlibs[j].library + k), topobject, key);
-      }
-   }
-   for (k = 0; k < xobjs.delbuffer.number; k++) {
-      searchinst(*(xobjs.delbuffer.library + k), topobject, key);
-   }
-
-   /* Also check through all instances on the library page */
-   if (l >= 0)
-      for (spec = xobjs.userlibs[l].instlist; spec != NULL; spec = spec->next) 
-         destroyinst(spec->thisinst, topobject, key);
-
-   free(ops->key);
-   free(ops);
-   
-   freelabel(subs);
-   incr_changes(topobject);
 }
 
 /*------------------------------------------------------*/
@@ -1461,7 +1541,7 @@ void unparameterize(int mode)
    else
       ptype = SEL_ANY;
 
-   if ((areastruct.selects == 1) && (ptype & LABEL) && textend > 0
+   if ((areastruct.selects == 1) && (mode == P_SUBSTRING) && textend > 0
 		&& textend < textpos) {
       if (SELECTTYPE(areastruct.selectlist) != LABEL) return;	 /* Not a label */
       settext = SELTOLABEL(areastruct.selectlist);
@@ -1486,7 +1566,7 @@ void unparameterize(int mode)
    else {
       for (fselect = areastruct.selectlist; fselect < areastruct.selectlist +
             areastruct.selects; fselect++) {
-         if ((ptype & LABEL) && SELECTTYPE(fselect) == LABEL) {
+         if ((mode == P_SUBSTRING) && SELECTTYPE(fselect) == LABEL) {
             settext = SELTOLABEL(fselect);
             strptr = settext->string;
             while (strptr != NULL && strptr->type != PARAM_START)
