@@ -409,7 +409,9 @@ extern void makenewencodingbutton(char *, char);
 extern void togglefontmark(int);
 #ifndef TCL_WRAPPER
 extern void toggleexcl(xcWidget, menuptr, int);
+extern void promptparam(xcWidget, caddr_t, caddr_t);
 #endif
+extern void stringparam(xcWidget, caddr_t, caddr_t);
 extern void toggle(xcWidget, Boolean *, caddr_t);
 extern void setcolorscheme(Boolean);
 extern void inversecolor(xcWidget, Boolean *, caddr_t);
@@ -528,20 +530,22 @@ extern oparamptr match_param(objectptr, char *);
 extern oparamptr match_instance_param(objinstptr, char *);
 extern oparamptr find_param(objinstptr, char *);
 extern int get_num_params(objectptr);
+extern void free_object_param(objectptr, oparamptr);
 extern void free_instance_param(objinstptr, oparamptr);
+extern void free_element_param(genericptr, eparamptr);
 
 extern oparamptr make_new_parameter(char *);
 extern eparamptr make_new_eparam(char *);
 
 extern void indicateparams(genericptr);
 extern void makenumericalp(genericptr *, u_int);
-extern void unmakenumericalp(genericptr *, u_int);
 extern void noparmstrcpy(u_char *, u_char *);
 extern void insertparam();
 extern void makeparam(labelptr, char *);
 extern void searchinst(objectptr, objectptr, char *);
 extern stringpart *searchparam(stringpart *);
 extern void unmakeparam(labelptr, stringpart *);
+extern void unmakenumericalp(genericptr *, u_int);         
 extern void unparameterize(int);
 extern void parameterize(int, char *);
 extern genericptr findparam(objectptr, void *, u_char);
