@@ -130,23 +130,10 @@ stringpart *makesegment(stringpart **strhead, stringpart *before)
 	 if (nextptr == before) {
 	    if (lastptr->type == PARAM_START) {
 	       oparamptr obs = NULL;
-	       int paramno = lastptr->data.paramno;
-	       if (areastruct.topinstance == NULL) {
-		  if (topobject->params != NULL)
-		     obs = topobject->params[paramno];
-		  else {
-		     Wprintf("Error:  Bad parameter!");
-		     break;
-		  }
-	       }
-	       else {
-	          if (areastruct.topinstance->params != NULL)
-	             obs = areastruct.topinstance->params[paramno];
-	          if (obs == NULL)
-		     obs = areastruct.topinstance->thisobject->params[paramno];
-	       }
+	       char *key = lastptr->data.string;
+	       obs = find_param(areastruct.topinstance, key);
 	       if (obs == NULL) {
-		  Wprintf("Error:  Bad parameter");
+		  Wprintf("Error:  Bad parameter!");
 		  return NULL;
 	       }
 	       obs->parameter.string = newptr;
@@ -216,7 +203,8 @@ stringpart *nextstringpart(stringpart *strptr, objinstptr thisinst)
 stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisinst)
 {
    stringpart *strptr, *nextptr;
-   int paramno;
+   char *key;
+   oparamptr ops;
 
    if (dstr == *strtop)
       *strtop = dstr->nextpart;
@@ -235,19 +223,14 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
       /* to the parameter in the parameter list, accordingly.		*/
 
       else if (strptr->type == PARAM_START) {
-	 paramno = strptr->data.paramno;
-	 if (thisinst == NULL)
-	 {
-	    if (topobject->params != NULL)
-	       topobject->params[paramno]->parameter.string = dstr->nextpart;
-	    else
-	       Fprintf(stderr, "Error in deletestring:  Bad parameter number found\n"); 
+	 key = strptr->data.string;
+	 ops = find_param(thisinst, key);
+	 if (ops == NULL) {
+	    Fprintf(stderr, "Error in deletestring:  Bad parameter %s found\n", key); 
+	 }
+	 else {
+	    ops->parameter.string = dstr->nextpart;
 	 }
-	 else if (thisinst->params == NULL || thisinst->params[paramno] == NULL)
-	    thisinst->thisobject->params[paramno]->parameter.string =
-			dstr->nextpart;
-	 else
-	    thisinst->params[paramno]->parameter.string = dstr->nextpart;
       }
       /* If this is the end of a parameter, we have to link the		*/
       /* PARAM_START, not the PARAM_END, which has already been nulled.	*/ 
@@ -301,51 +284,36 @@ stringpart *mergestring(stringpart *firststr)
 
 stringpart *linkstring(objinstptr localinst, stringpart *strstart)
 {
-   int paramno;
+   char *key;
    stringpart *nextptr = NULL, *tmpptr;
+   oparamptr ops;
 
    if (strstart->type != PARAM_START) return NULL;
 
-   paramno = strstart->data.paramno;
+   key = strstart->data.string;
 
    /* In case of no calling instance, always get the default from the	*/
    /* current page object.						*/
 
    if (localinst == NULL) {
-      if (topobject->num_params > paramno) {
-         if (topobject->params[paramno]->type != XC_STRING)
-	    goto linkerror;
-	 else
-            nextptr = topobject->params[paramno]->parameter.string;
-      }
-      else
-	 return NULL;  /* There is no parameter here. */
+      ops = match_param(topobject, key);
+      if (ops == NULL) return NULL;
    }
    else {
-      if (localinst->params != NULL) {
-	 if (localinst->params[paramno] != NULL) {
-	    if (localinst->params[paramno]->type != XC_STRING)
-	       goto linkerror;
-            nextptr = localinst->params[paramno]->parameter.string;
-	 }
-      }
-
-      /* If there's no instance, substitute the default parameter */
-
-      if (nextptr == NULL) {
-	 if (localinst->thisobject->params != NULL) {
-	    if (localinst->thisobject->params[paramno]->type != XC_STRING)
-	       goto linkerror;
-            nextptr = localinst->thisobject->params[paramno]->parameter.string;
-	 }
-	 else
-	    /* We get here in cases where the object definition is being read,	*/
-	    /* and there is no instance of the object to link to.  In that	*/
-	    /* case, we ignore parameters and move on to the next part.		*/
-	    return strstart->nextpart;
+      ops = find_param(localinst, key);
+      if (ops == NULL) {
+	 /* We get here in cases where the object definition is being read,	*/
+	 /* and there is no instance of the object to link to.  In that	*/
+	 /* case, we ignore parameters and move on to the next part.		*/
+	 return strstart->nextpart;
       }
    }
 
+   if (ops->type != XC_STRING)
+      goto linkerror;
+   else
+      nextptr = ops->parameter.string;
+
    /* If the parameter exists, link the end of the parameter back to	*/
    /* the calling string.						*/
 
@@ -360,8 +328,7 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart)
    return NULL;
 
 linkerror:
-   Fprintf(stderr, "linkstring() error: parameter %d is not a substring!\n",
-		paramno);
+   Fprintf(stderr, "linkstring() error: parameter %s is not a substring!\n", key);
    return strstart->nextpart;
 }
 
@@ -460,7 +427,7 @@ void charprint(char *sout, stringpart *strptr, int locpos)
 	 sprintf(sout, "Kern=(%d,%d)", strptr->data.kern[0], strptr->data.kern[1]);
 	 break;
       case PARAM_START:
-	 sprintf(sout, "Parameter(%d)<", strptr->data.paramno + 1);
+	 sprintf(sout, "Parameter(%s)<", strptr->data.string);
 	 break;
       default:
          strcpy(sout, nonprint[strptr->type]);
@@ -766,7 +733,8 @@ stringpart *stringcopyall(stringpart *string, objinstptr thisinst)
       if (strptr == string) newtop = newpart;
       else topend->nextpart = newpart;
       topend = newpart;
-      if ((strptr->type == TEXT_STRING) && strptr->data.string) {
+      if ((strptr->type == TEXT_STRING || strptr->type == PARAM_START)
+		&& strptr->data.string) {
 	 newpart->data.string = (char *)malloc(1 + strlen(strptr->data.string));
 	 strcpy(newpart->data.string, strptr->data.string);
       }
@@ -784,7 +752,7 @@ stringpart *stringcopyall(stringpart *string, objinstptr thisinst)
 stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 {
    stringpart *strptr, *newpart, *curend = NULL, *rettop, *curtop, *savend;
-   int paramno = -1;
+   char *key = NULL;
    oparamptr pparam;
 
    for (strptr = string; strptr != NULL; strptr = strptr->nextpart) {
@@ -798,7 +766,7 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 
       if (curend) {
          if (curend->type == PARAM_START) {
-	    paramno = curend->data.paramno;
+	    key = curend->data.string;
 	    curtop = newpart;
 	    savend = curend;
          }
@@ -809,7 +777,7 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
       }
       curend = newpart;
 
-      if (strptr->type == TEXT_STRING) {
+      if (strptr->type == TEXT_STRING || strptr->type == PARAM_START) {
 	 if (strptr->data.string) {
 	    newpart->data.string = (char *)malloc(1 + strlen(strptr->data.string));
 	    strcpy(newpart->data.string, strptr->data.string);
@@ -820,21 +788,16 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
       else if (strptr->type == PARAM_END) {
 	 /* If thisinst is NULL, then use topobject (top level object) */
 
-	 if (paramno >= 0) {
-	    if (thisinst == NULL) {
-	       if (topobject->params != NULL)
-	          pparam = topobject->params[paramno];
-	       else
-		  Fprintf(stderr, "Error:  Bad parameter number encountered!\n");
+	 if (key != NULL) {
+	    pparam = find_param(thisinst, key);
+	    if (pparam == NULL) {
+	       Fprintf(stderr, "Error:  Bad parameter %s encountered!\n", key);
+	    }
+	    else  {
+	       freelabel(pparam->parameter.string);
+	       pparam->parameter.string = curtop;
+	       key = NULL;
 	    }
-	    else if ((thisinst->params) != NULL && (thisinst->params[paramno] != NULL))
-	       pparam = thisinst->params[paramno];
-	    else 
-	       pparam = thisinst->thisobject->params[paramno];
-
-	    freelabel(pparam->parameter.string);
-	    pparam->parameter.string = curtop;
-	    paramno = -1;
 	 }
          else {
 	    Fprintf(stderr, "Error:  Bad parameter in stringcopyback()\n");
